/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "recent.h"

static int priv_recent_info_free(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting a GtkRecentInfo!");
	#endif
	
	Object* obj = lua_touserdata(L, 1);
	gtk_recent_info_unref((GtkRecentInfo*)obj->pointer);
	
	return 0;
}

static int priv_recent_info_new(lua_State* L, GtkRecentInfo* ptr)
{
	if(ptr)
	{
		Object* obj = (Object*)lua_newuserdata(L, sizeof(Object));
		obj->pointer = ptr;
		obj->need_unref = TRUE; /* Just for consistency */
		int top = lua_gettop(L);
		
		/* Shared metatable */
		if(luaL_newmetatable(L, "__lguiReCeNtInFo"))
		{
			lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
			lua_pushvalue(L, -2);
			lua_pushstring(L, "lgui");
			lua_pushstring(L, "RecentInfo");
			lua_pushcfunction(L, priv_recent_info_free);
			lua_pushcfunction(L, lgui_recent_info_get_short_name);
			lua_call(L, 5, 0);
		}

		/* Set the object metatable */
		lua_setmetatable(L, top);
	
		return TRUE;
	}
	
	lua_pushnil(L);
		
	return FALSE;
}

static int priv_recent_info_handler(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Handling a GtkRecentInfo!");
	#endif
	
	GtkRecentInfo* ptr = lua_touserdata(L, -1);
	priv_recent_info_new(L, ptr);
	
	return 1;
}

int lgui_recent_manager_new(lua_State* L)
{
	GtkRecentManager* ptr = gtk_recent_manager_new();
	object_new(L, ptr, TRUE);
	return 1;
}

int lgui_recent_manager_get_default(lua_State* L)
{
	GtkRecentManager* ptr = gtk_recent_manager_get_default();
	object_new(L, ptr, FALSE);
	return 1;
}

int lgui_recent_manager_add_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_recent_manager_add_item(GTK_RECENT_MANAGER(obj->pointer), lua_tostring(L, 2));
	lua_pushboolean(L, res);
	return 1;
}

int lgui_recent_manager_remove_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	GError* errors = NULL;
	gboolean res = gtk_recent_manager_remove_item(GTK_RECENT_MANAGER(obj->pointer),
		lua_tostring(L, 2), &errors);
		
	lua_pushboolean(L, res);
	
	if(errors)
	{
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
	
	return 1;
}

int lgui_recent_manager_lookup_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	GError* errors = NULL;
	GtkRecentInfo* ptr = gtk_recent_manager_lookup_item(GTK_RECENT_MANAGER(obj->pointer),
		lua_tostring(L, 2), &errors);
	
	if(errors)
	{
		lua_pushboolean(L, FALSE);
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
	
	priv_recent_info_new(L, ptr);	
	return 1;
}

int lgui_recent_manager_has_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);	
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_recent_manager_has_item(GTK_RECENT_MANAGER(obj->pointer),
		lua_tostring(L, 2));
		
	lua_pushboolean(L, res);
	return 1;
}

int lgui_recent_manager_move_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	GError* errors = NULL;
	gboolean res = gtk_recent_manager_move_item(GTK_RECENT_MANAGER(obj->pointer),
		lua_tostring(L, 2), lua_tostring(L, 3), &errors);
		
	lua_pushboolean(L, res);
	
	if(errors)
	{
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
	
	return 1;
}

int lgui_recent_manager_purge_items(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GError* errors = NULL;
	gint res = gtk_recent_manager_purge_items(GTK_RECENT_MANAGER(obj->pointer),
		&errors);
		
	if(errors)
	{
		lua_pushboolean(L, FALSE);
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
		
	lua_pushnumber(L, res);
	
	return 1;
}

int lgui_recent_manager_get_items(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GList* items = gtk_recent_manager_get_items(GTK_RECENT_MANAGER(obj->pointer));
	lua_pushstring(L, "__lobjGList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, items);
	lua_pushinteger(L, TYPE_CUSTOM);
	lua_pushcfunction(L, priv_recent_info_handler);
	lua_call(L, 3, 1);
	
	/* Free the list (the content will be Garbage-collected) */
	g_list_free(items);

	return 1;
}

/* GtkRecentInfo */

int lgui_recent_info_get_uri(lua_State* L)
{	
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	const gchar* res = gtk_recent_info_get_uri((GtkRecentInfo*)obj->pointer);
	lua_pushstring(L, res);
	
	return 1;
}

int lgui_recent_info_get_display_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	const gchar* res = gtk_recent_info_get_display_name((GtkRecentInfo*)obj->pointer);
	lua_pushstring(L, res);
	
	return 1;
}

int lgui_recent_info_get_description(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	const gchar* res = gtk_recent_info_get_description((GtkRecentInfo*)obj->pointer);
	lua_pushstring(L, res);
	
	return 1;
}

int lgui_recent_info_get_mime_type(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	const gchar* res = gtk_recent_info_get_mime_type((GtkRecentInfo*)obj->pointer);
	lua_pushstring(L, res);
	
	return 1;
}

int lgui_recent_info_get_added(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	time_t res = gtk_recent_info_get_added((GtkRecentInfo*)obj->pointer);
	lua_pushnumber(L, res);
	
	return 1;
}

int lgui_recent_info_get_modified(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	time_t res = gtk_recent_info_get_modified((GtkRecentInfo*)obj->pointer);
	lua_pushnumber(L, res);
	
	return 1;
}

int lgui_recent_info_get_visited(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	time_t res = gtk_recent_info_get_visited((GtkRecentInfo*)obj->pointer);
	lua_pushnumber(L, res);
	
	return 1;
}

int lgui_recent_info_get_private_hint(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gboolean res = gtk_recent_info_get_private_hint((GtkRecentInfo*)obj->pointer);
	lua_pushboolean(L, res);
	
	return 1;
}

int lgui_recent_info_get_short_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gchar* res = gtk_recent_info_get_short_name((GtkRecentInfo*)obj->pointer);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lgui_recent_info_last_application(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gchar* res = gtk_recent_info_last_application((GtkRecentInfo*)obj->pointer);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lgui_recent_info_exists(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_recent_info_exists((GtkRecentInfo*)obj->pointer);
	lua_pushboolean(L, res);
	
	return 1;
}

int lgui_recent_info_get_applications(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gsize length;
	GStrv files = gtk_recent_info_get_applications((GtkRecentInfo*)obj->pointer,
		&length);
	lua_pushstring(L, "__lobjGstrv2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, files);
	lua_pushnumber(L, length);
	lua_call(L, 2, 1);

	g_strfreev(files);
	
	return 1;
}

/* GtkRecentFilter */

int lgui_recent_filter_new(lua_State* L)
{
	GtkRecentFilter* ptr = gtk_recent_filter_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_recent_filter_get_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	const gchar* ptr = gtk_recent_filter_get_name(GTK_RECENT_FILTER(obj->pointer));
	lua_pushstring(L, ptr);
	
	return 1;
}

int lgui_recent_filter_set_name(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_filter_set_name(GTK_RECENT_FILTER(obj->pointer), lua_tostring(L, 2));
	
	return 0;
}

int lgui_recent_filter_add_mime_type(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_filter_add_mime_type(GTK_RECENT_FILTER(obj->pointer), lua_tostring(L, 2));
	
	return 0;
}

int lgui_recent_filter_add_pattern(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_filter_add_pattern(GTK_RECENT_FILTER(obj->pointer), lua_tostring(L, 2));
	
	return 0;
}

int lgui_recent_filter_add_application(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_filter_add_application(GTK_RECENT_FILTER(obj->pointer), lua_tostring(L, 2));
	
	return 0;
}

int lgui_recent_filter_add_age(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_filter_add_age(GTK_RECENT_FILTER(obj->pointer), lua_tointeger(L, 2));
	
	return 0;
}

int lgui_recent_filter_add_pixbuf_formats(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_filter_add_pixbuf_formats(GTK_RECENT_FILTER(obj->pointer));
	
	return 0;
}

/* GtkRecentChooser */

int lgui_recent_chooser_add_filter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 1);
	
	gtk_recent_chooser_add_filter(GTK_RECENT_CHOOSER(obj1->pointer),
		GTK_RECENT_FILTER(obj2->pointer));
		
	return 0;
}

int lgui_recent_chooser_remove_filter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 1);
	
	gtk_recent_chooser_remove_filter(GTK_RECENT_CHOOSER(obj1->pointer),
		GTK_RECENT_FILTER(obj2->pointer));
		
	return 0;
}

int lgui_recent_chooser_select_uri(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	GError* errors = NULL;
	gboolean res = gtk_recent_chooser_select_uri(GTK_RECENT_CHOOSER(obj->pointer),
		lua_tostring(L, 2), &errors);
	
	lua_pushboolean(L, res);
	
	if(errors)
	{
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
		
	return 1;
}

int lgui_recent_chooser_unselect_uri(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_chooser_unselect_uri(GTK_RECENT_CHOOSER(obj->pointer),
		lua_tostring(L, 2));
		
	return 0;
}

int lgui_recent_chooser_select_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_chooser_select_all(GTK_RECENT_CHOOSER(obj->pointer));
	return 0;
}

int lgui_recent_chooser_unselect_all(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_recent_chooser_unselect_all(GTK_RECENT_CHOOSER(obj->pointer));
	return 0;
}

int lgui_recent_chooser_set_current_uri(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	GError* errors = NULL;
	gboolean res = gtk_recent_chooser_set_current_uri(GTK_RECENT_CHOOSER(obj->pointer),
		lua_tostring(L, 2), &errors);
	
	lua_pushboolean(L, res);
	
	if(errors)
	{
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
		
	return 1;
}

int lgui_recent_chooser_get_current_uri(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gchar* ptr = gtk_recent_chooser_get_current_uri(GTK_RECENT_CHOOSER(obj->pointer));
	lua_pushstring(L, ptr);
	g_free(ptr);
		
	return 1;
}

int lgui_recent_chooser_get_current_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkRecentInfo* ptr = gtk_recent_chooser_get_current_item(GTK_RECENT_CHOOSER(obj->pointer));
	priv_recent_info_new(L, ptr);
		
	return 1;
}

int lgui_recent_chooser_get_items(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GList* items = gtk_recent_chooser_get_items(GTK_RECENT_CHOOSER(obj->pointer));
	lua_pushstring(L, "__lobjGList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, items);
	lua_pushinteger(L, TYPE_CUSTOM);
	lua_pushcfunction(L, priv_recent_info_handler);
	lua_call(L, 3, 1);
	
	/* Free the list (the content will be Garbage-collected) */
	g_list_free(items);

	return 1;
}

/* GtkRecentChooserDialog */

int lgui_recent_chooser_dialog_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TSTRING);
	luaL_checktype(L, 4, LUA_TNUMBER);
	
	int size = lua_gettop(L), i;	
	GtkWindow* parent = NULL;
	
	if(lua_type(L, 2) == LUA_TUSERDATA)
	{		
		Object* obj = lua_touserdata(L, 2);
		parent = GTK_WINDOW(obj->pointer);
	}

	GtkWidget* ptr = gtk_recent_chooser_dialog_new(lua_tostring(L, 1), parent,
		lua_tostring(L, 3), lua_tointeger(L, 4), NULL);

	/* The buttons */
	for(i = 5; i <= size; i += 2)
	{
		luaL_checktype(L, i, LUA_TSTRING);
		luaL_checktype(L, i + 1, LUA_TNUMBER);
		gtk_dialog_add_button(GTK_DIALOG(ptr), lua_tostring(L, i), lua_tonumber(L, i + 1));
	}

	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_recent_chooser_dialog_new_for_manager(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	luaL_checktype(L, 4, LUA_TSTRING);
	luaL_checktype(L, 5, LUA_TNUMBER);
	
	Object* obj2 = lua_touserdata(L, 3);
	GtkWindow* parent = NULL;
	int size = lua_gettop(L), i;
	
	if(lua_type(L, 2) == LUA_TUSERDATA)
	{		
		Object* obj = lua_touserdata(L, 2);
		parent = GTK_WINDOW(obj->pointer);
	}

	GtkWidget* ptr = gtk_recent_chooser_dialog_new_for_manager(lua_tostring(L, 1),
		parent, GTK_RECENT_MANAGER(obj2->pointer), lua_tostring(L, 4), lua_tointeger(L, 5), NULL);

	/* The buttons */
	for(i = 6; i <= size; i += 2)
	{
		luaL_checktype(L, i, LUA_TSTRING);
		luaL_checktype(L, i + 1, LUA_TNUMBER);
		gtk_dialog_add_button(GTK_DIALOG(ptr), lua_tostring(L, i), lua_tonumber(L, i + 1));
	}

	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkRecentChooserMenu */

int lgui_recent_chooser_menu_new(lua_State* L)
{
	GtkWidget* ptr = gtk_recent_chooser_menu_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_recent_chooser_menu_new_for_manager(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_recent_chooser_menu_new_for_manager(GTK_RECENT_MANAGER(obj->pointer));
	object_new(L, ptr, FALSE);
	
	return 1;
}
